﻿//Mr.GearBuyer - Created by CodenameGamma - 12/26/11 - For WoW Version 4.0.3+
//Updated for Mist of Panderia WoW Version 5.0.4+ on 9/12/12
//Refactored and updated by RichieSams - For WoW Version 5.1+ on 12/10/12
//www.honorbuddy.com
//This is a free plugin, and should not be sold, or repackaged.
//Donations Accepted. 
//Version 2.7

using System;
using System.ComponentModel;
using System.Windows.Forms;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System.IO;
using System.Xml.Linq;
using Styx.Plugins;
using Styx;
using Styx.CommonBot;
using Styx.Common.Helpers;
using Styx.CommonBot.POI;
using Styx.CommonBot.Profiles;
using Styx.CommonBot.Database;
using Styx.Common;
using Styx.CommonBot.Frames;
using Styx.MemoryManagement;

namespace MrGearBuyer.GUI
{
    public partial class MGBConfig : Form
    {
        #region Variables

        // List to hold what a specific vendor has to offer
        private static BindingList<ItemInfo> vendorList = new BindingList<ItemInfo>();

        #endregion

        public MGBConfig()
        {
            InitializeComponent();

            dgvBuyList.DataSource = MrGearBuyer.wishList;
            dgvVenderList.DataSource = vendorList;

            LogoutAtHonorCap_Chkbox.Checked = MrGearBuyer.LogoutAtHonorCap;
            LogoutAtJusticeCap_Chkbox.Checked = MrGearBuyer.LogoutAtJusticeCap;
            RemoveHPJPWhenCap_Chkbox.Checked = MrGearBuyer.RemoveHPJPWhenCapped;
        }

        private void dgvBuyList_MouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            RemoveBuyClick(null, null);
        }

        private void RemoveBuyClick(object sender, EventArgs e)
        {
            foreach (DataGridViewRow row in dgvBuyList.SelectedRows)
            {
                MrGearBuyer.wishList.RemoveAt(row.Index);
            }
        }

        private void BtnMoveUpClick(object sender, EventArgs e)
        {
            int count = dgvBuyList.SelectedRows.Count;

            if (count == 0)
            {
                MessageBox.Show("Please select a row first", "Select a row", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                return;
            }

            if (count >= 2)
            {
                MessageBox.Show("Please select only one row to move it up", "Select only one row", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                return;
            }
            
            int index = dgvBuyList.SelectedRows[0].Index;

            // Don't do anything if it's already the first row.
            if (index == 0)
                return;

            ItemInfo item = MrGearBuyer.wishList[index];

            
            MrGearBuyer.wishList.RemoveAt(index);

            // Decrement the index
            index--;

            
            MrGearBuyer.wishList.Insert(index, item);

            foreach (DataGridViewRow row in dgvBuyList.Rows)
            {
                row.Selected = false;
            }

            dgvBuyList.Rows[index].Selected = true;
        }

        private void BtnMoveDownClick(object sender, EventArgs e)
        {
            int count = dgvBuyList.SelectedRows.Count;

            if (count == 0)
            {
                MessageBox.Show("Please select a row first", "Select a row", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                return;
            }

            if (count >= 2)
            {
                MessageBox.Show("Please select only one row to move it down", "Select only one row", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                return;
            }

            int index = dgvBuyList.SelectedRows[0].Index;

            // Don't do anything if it's already the last row.
            if (index == 
                MrGearBuyer.wishList.Count - 1)
                return;

            ItemInfo item = MrGearBuyer.wishList[index];

            
            MrGearBuyer.wishList.RemoveAt(index);

            // Increment the index
            index++;

            MrGearBuyer.wishList.Insert(index, item);

            foreach (var row in dgvBuyList.Rows)
            {
                ((DataGridViewRow)row).Selected = false;
            }

            dgvBuyList.Rows[index].Selected = true;
        }

        private void dgvVenderList_MouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            AddtoBuyClick(null, null);
        }

        private void AddtoBuyClick(object sender, EventArgs e)
        {
            foreach (DataGridViewRow item in dgvVenderList.SelectedRows)
            {
                MrGearBuyer.wishList.Add((ItemInfo)(item.DataBoundItem));
            }
        }

        private void FetchClick(object sender, EventArgs e)
        {
            ObjectManager.Update();

            // Only fetch new items if a vendor frame is open
            if (!MerchantFrame.Instance.IsVisible)
            {
                MrGearBuyer.Slog("No vendor frame is open. Talk to a vendor to see their goods, then try again.");
                return;
            }

            // Make way for new items
            vendorList.Clear();

            uint vendorID = StyxWoW.Me.CurrentTarget.Entry;

            foreach (MerchantItem vendorItem in MerchantFrame.Instance.GetAllMerchantItems())
            {
                int fixedIndex = vendorItem.Index + 1;
                String name = vendorItem.Name;

                // If the core fails at getting the name, try to do it manually
                if (name == "(null)")
                {
                    // Buying HP with JP
                    if (vendorItem.ItemId == 392)
                        name = "Honor Points";
                    // Buying JP with HP
                    else if (vendorItem.ItemId == 395)
                        name = "Justice Points";
                    else
                    {
                        // Core fails. Manual look-up
                        name = Lua.GetReturnVal<String>(
                               "return GetMerchantItemInfo(" + fixedIndex + ")", 0);

                        // If it still fails, skip it
                        if (name == "(null)")
                        {
                            MrGearBuyer.Slog("Can not find the name for itemID [{0}]. Please inform the plugin writers and attach your log file", vendorItem.ItemId);
                            continue;
                        }
                    }
                }          
                
                String costTypeLua = Lua.GetReturnVal<string>(
                                     "return GetMerchantItemCostItem(" + fixedIndex + ", GetMerchantItemCostInfo(" + fixedIndex + "))", 0);

                if (string.IsNullOrEmpty(costTypeLua))
                {
                    MrGearBuyer.Dlog("Vender item is not bought with points, Skipping. [{0}]", name);
                    continue;
                }

                ItemInfo newItem = new ItemInfo();

                // Set the name and item cost type
                if (costTypeLua.ToUpperInvariant().Contains("HONOR"))
                    newItem.ItemCostType = WoWCurrencyType.HonorPoints;
                else if (costTypeLua.ToUpperInvariant().Contains("JUSTICE"))
                    newItem.ItemCostType = WoWCurrencyType.JusticePoints;
                else if (costTypeLua.ToUpperInvariant().Contains("CONQUEST"))
                    newItem.ItemCostType = WoWCurrencyType.ConquestPoints;
                else
                {
                    // Item we don't care about
                    MrGearBuyer.Dlog("Vendor item is not bought with honor/justice/conquest points. [{0}]", name);
                    continue;
                }

                int cost = Lua.GetReturnVal<int>(
                           "return GetMerchantItemCostItem(" + fixedIndex + ", GetMerchantItemCostInfo(" + fixedIndex + "))", 1);

                // Sometimes name returns empty from lua. Do another one just in case
                if (cost == 0)
                {
                    cost = Lua.GetReturnVal<int>(
                           "return GetMerchantItemCostItem(" + fixedIndex + ", GetMerchantItemCostInfo(" + fixedIndex + "))", 1);
                    if (cost == 0)
                    {
                        MrGearBuyer.Dlog("Vender item not usable, Skipping. [{0}]", name);
                        continue;
                    }
                }

                

                newItem.ItemId = vendorItem.ItemId;
                newItem.ItemName = name;
                newItem.ItemCost = (uint)cost;
                newItem.ItemSupplierId = vendorID;
                
                vendorList.Add(newItem);
            }
        }

        private void RemoveJphpWhenCappedCheckedChanged(object sender, EventArgs e)
        {
            MrGearBuyer.RemoveHPJPWhenCapped = RemoveHPJPWhenCap_Chkbox.Checked;
        }

        private void RemoveFromBL_Click(object sender, EventArgs e)
        {
            if(StyxWoW.Me.CurrentTarget != null)
            {
                // We need to make sure vendor is usable, so remove any blacklists.
                if (Blacklist.Contains(StyxWoW.Me.CurrentTarget))
                {
                    MrGearBuyer.Slog("{0} was Found to be Blacklisted, Clearing Blacklist.", StyxWoW.Me.CurrentTarget.Name);
                    Blacklist.Flush();
                }
            }
        }

        private void LogoutAtHonorCap_Chkbox_CheckedChanged(object sender, EventArgs e)
        {
            MrGearBuyer.LogoutAtHonorCap = LogoutAtHonorCap_Chkbox.Checked;
        }

        private void LogoutAtJusticeCap_Chkbox_CheckedChanged(object sender, EventArgs e)
        {
            MrGearBuyer.LogoutAtJusticeCap = LogoutAtJusticeCap_Chkbox.Checked;
        }

        private void RemoveHPJPWhenCap_Chkbox_CheckedChanged(object sender, EventArgs e)
        {
            MrGearBuyer.RemoveHPJPWhenCapped = RemoveHPJPWhenCap_Chkbox.Checked;
        }

        private void saveSettingsButton_Click(object sender, EventArgs e)
        {
            MrGearBuyer.SaveSettings();
        }

        private void MGBConfig_FormClosing(object sender, FormClosingEventArgs e)
        {
            MrGearBuyer.SaveSettings();
        }
    }
}
